package sim;

import java.awt.FlowLayout;
import java.awt.Panel;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import sim.util.SimButton;
import sim.util.SimButtonListener;

public class RandomShortcut extends Panel {

	public static boolean RANDOM = false;
	private RandomShortcutListener listener;

	public RandomShortcut() {
		super();
		this.setLayout(new FlowLayout(FlowLayout.LEFT, 8, 0));
	}

	public void addNotify() {
		super.addNotify();
		this.listener = new RandomShortcutListener(this);
	}
	/* ==================================================================
	 Services provided to Menu
	 ================================================================ */
	
	public void setRandom(boolean israndom) {
		this.listener.setRandom(israndom);
	}

	/* ==================================================================
	 Listener to handle mouse events
	 ================================================================= */
	private class RandomShortcutListener extends SimButtonListener {
		private SimButton random;
		private boolean buttonUpOnLastMouseDown;

		public RandomShortcutListener(RandomShortcut parent) {
			Panel p = new Panel(new FlowLayout(FlowLayout.LEFT, 0, 0));
			parent.add(p);

			this.buttonUpOnLastMouseDown = true;
			this.random = new SimButton("sim/images/Random.png", "Random delay", NorthPanel.BUBBLE_WAIT);
			this.random.setListener(this);
			p.add(this.random);
		}

		public void actionPerformed(ActionEvent e) {
			SimMenuBar.RANDOM.setState(this.random.isInset());
			RANDOM = this.random.isInset();
		}

		public void activate(SimButton ib) {
			ib.processActionEvent();
			ib.setArmed(false);
		}

		public void arm(SimButton ib) {
			ib.setArmed(true);
		}

		public void disarm(SimButton ib) {
			ib.setArmed(false);
		}

		public void mousePressed(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled()) {
				if (ib.isRaised()) {
					ib.paintInset();
					this.buttonUpOnLastMouseDown = false;
					this.arm(ib);
				} else {
					ib.paintRaised();
					this.buttonUpOnLastMouseDown = true;
					this.arm(ib);
				}

			}
		}

		public void mouseClicked(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled() && ib.isArmed()) {
				this.activate(ib);
			}
		}

		public void mouseReleased(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			Point pt = event.getPoint();

			if (ib.contains(pt.x, pt.y)) {
				this.mouseClicked(event);
			}
		}

		public void mouseDragged(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled() && ib.isArmed()) {
				Point loc = event.getPoint();

				if (ib.contains(loc.x, loc.y)) {
					if (this.buttonUpOnLastMouseDown) {
						if (!ib.isRaised()) {
							ib.paintRaised();
						}
					} else if (ib.isRaised()) {
						ib.paintInset();
					}
				} else {
					if (this.buttonUpOnLastMouseDown) {
						if (ib.isRaised()) {
							ib.paintInset();
						}
					} else if (!ib.isRaised()) {
						ib.paintRaised();
					}
				}
			}
		}

		public void setRandom(boolean israndom) {
			if (israndom) {
				this.random.paintInset();
			} else {
				this.random.paintRaised();
			}
			RANDOM =  this.random.isInset();
		}
	}
}