package sim;

import java.awt.*;
import java.awt.event.*;

import sim.util.DialogButton;
import sim.util.ImagePanel;
import sim.util.SimSeparator;

public class VHDLExportChoices extends Dialog implements ActionListener {

	public static Color LED_COLOR = SimulationProperties.ONE_COLOR;
	public static Color SW_COLOR = SimulationProperties.ONE_COLOR;
	public static Color PB_COLOR = SimulationProperties.ONE_COLOR;
	public static Color ND_COLOR = SimulationProperties.ONE_COLOR;
	public static double FROM = 100000000;
	public static double TO = 1;
	private static Image ICON;
	
	public static String getVoltage(Color c) {
		String result = "no";

		if (c.getRGB() == SimulationProperties.ONE_COLOR.getRGB()) {
			result = "high";
		} else if (c.getRGB() == SimulationProperties.ZERO_COLOR.getRGB()) {
			result = "low";
		}

		return result;

	}

	public static Color getColor(String voltage) {
		Color result = Color.black;

		if (voltage.equals("high")) {
			result = SimulationProperties.ONE_COLOR;
		} else if (voltage.equals("low")) {
			result = SimulationProperties.ZERO_COLOR;
		}

		return result;
	}
	
	private ActualProperties properties;
	private ImagePanel iconHolder;

	public VHDLExportChoices() {
		super(MainWindow.MAIN_WINDOW, "Properties");

		if (VHDLExportChoices.ICON == null) {
			VHDLExportChoices.ICON = GuiFileLink.getImage("sim/images/VHDLExportChoices.png");
		}

		this.setLayout(new GridBagLayout());
		this.setModal(true);
		this.setResizable(false);

		Panel p = new Panel(new FlowLayout(FlowLayout.RIGHT, 10, 10));

		DialogButton b = new DialogButton("OK");
		b.addActionListener(this);
		p.add(b);
		b = new DialogButton("Cancel");
		b.addActionListener(this);
		p.add(b);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;

		this.add(p, c);

		this.properties = new ActualProperties(FROM, TO);

		c.gridy = 1;
		c.insets = new Insets(10, 20, 40, 20);
		c.anchor = GridBagConstraints.CENTER;
		this.add(this.properties, c);

		p = new Panel(new FlowLayout(FlowLayout.LEFT, 10, 0));
		this.iconHolder = new ImagePanel(VHDLExportChoices.ICON);
		p.add(this.iconHolder);
		p.add(new Label("Export Choices", Label.LEFT));

		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridy = 0;

		this.add(p, c);

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				setVisible(false);
			}
		});
	}

	public void setVisible(boolean visible) {
		if (visible) {
			this.properties.initialise();
		}

		super.setVisible(visible);
	}

	public void addNotify() {
		super.addNotify();

		int width = 40 + this.properties.getSize().width + this.getInsets().left + this.getInsets().right;
		int height = 70 + this.iconHolder.getPreferredSize().height + this.properties.getSize().height + DialogButton.HEIGHT + this.getInsets().top + this.getInsets().bottom;
		this.setSize(width, height);
		doLayout();

		this.setLocation(MainWindow.getMiddleOfScreen(this));
	}

	public void actionPerformed(ActionEvent e) {
		this.setVisible(false);

		if (((DialogButton) e.getSource()).getLabel().equals("OK")) {
			this.properties.ok();
		}
	}

	public void paint(Graphics g) {
		super.paint(g);

		Rectangle r = this.properties.getBounds();

		r.x = r.x - 10;
		r.y = r.y - 5;

		r.width = r.width + 20;
		r.height = r.height + 40;

		g.setColor(Color.white);
		g.drawLine(r.x, r.y, r.x + r.width - 2, r.y);
		g.drawLine(r.x, r.y, r.x, r.y + r.height - 2);

		g.setColor(Color.black);
		g.drawLine(r.x + r.width - 1, r.y, r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine(r.x, r.y + r.height - 1, r.x + r.width - 1, r.y + r.height - 1);

		g.setColor(this.getBackground().darker());
		g.drawLine(r.x + r.width - 2, r.y + 1, r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine(r.x + 1, r.y + r.height - 2, r.x + r.width - 2, r.y + r.height - 2);
	}

	private class ActualProperties extends Container implements ActionListener, FocusListener, ItemListener, AdjustmentListener {

		private Choice editLED = new Choice();
		private Choice editSW = new Choice();
		private Choice editPB = new Choice();
		private Choice editND = new Choice();
		private TextField editFrom = new TextField();
		private TextField editTo = new TextField();
		private double oldfrom;
		private double oldto;
		private Color oldLED;
		private Color oldSW;
		private Color oldPB;
		private Color oldND;

		private Label activevoltage = new Label("Active Voltage");
		private Label clock = new Label("Clock");
		
		public ActualProperties(double from, double to) {
			super();
			this.setLayout(new BorderLayout(0, 15));

			this.editLED.add("high");
			this.editLED.add("low");
			this.editLED.addItemListener(this);
			this.editLED.addFocusListener(this);

			this.editSW.add("high");
			this.editSW.add("low");
			this.editSW.addItemListener(this);
			this.editSW.addFocusListener(this);

			this.editPB.add("high");
			this.editPB.add("low");
			this.editPB.addItemListener(this);
			this.editPB.addFocusListener(this);
			
			this.editND.add("high");
			this.editND.add("low");
			this.editND.addItemListener(this);
			this.editND.addFocusListener(this);

			this.editFrom.addActionListener(this);
			this.editFrom.addFocusListener(this);
			this.editFrom.setText(Double.toString(from));

			this.editTo.addActionListener(this);
			this.editTo.addFocusListener(this);
			this.editTo.setText(Double.toString(to));
		}

		public void addNotify() {
			super.addNotify();
			this.setSize(290, this.editLED.getPreferredSize().height * 3 + this.activevoltage.getPreferredSize().height * 2 + 65);
		}

		public void initialise() {
			this.oldLED = VHDLExportChoices.LED_COLOR;
			this.oldSW = VHDLExportChoices.SW_COLOR;
			this.oldPB = VHDLExportChoices.PB_COLOR;
			this.oldND = VHDLExportChoices.ND_COLOR;
			this.oldfrom = VHDLExportChoices.FROM;
			this.oldto = VHDLExportChoices.TO;

			this.editLED.select(VHDLExportChoices.getVoltage(this.oldLED));
			this.editSW.select(VHDLExportChoices.getVoltage(this.oldSW));
			this.editPB.select(VHDLExportChoices.getVoltage(this.oldPB));
			this.editND.select(VHDLExportChoices.getVoltage(this.oldND));
			this.editFrom.setText(Double.toString(this.oldfrom));
			this.editTo.setText(Double.toString(this.oldto));

			this.editLED.setBackground(this.oldLED);
			this.editSW.setBackground(this.oldSW);
			this.editPB.setBackground(this.oldPB);
			this.editND.setBackground(this.oldND);

			Panel big = new Panel(new BorderLayout(0, 15));
			Panel p = new Panel(new BorderLayout());
			p.add(this.activevoltage, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);

			Panel p2 = new Panel(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			p = new Panel(new BorderLayout());

			c.weightx = 1;
			c.weighty = 0;
			c.gridheight = 1;
			c.fill = GridBagConstraints.NONE;

			c.insets = new Insets(0, 0, 0, 0);
			c.gridwidth = 1;
			c.gridy = 1;
			c.anchor = GridBagConstraints.EAST;
			p2.add(new Label("LEDs: "), c);

			c.gridx = 1;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editLED, c);
			p.add(p2,BorderLayout.WEST);

			c.gridx = 2;
			c.anchor = GridBagConstraints.EAST;
			
			p2 = new Panel(new GridBagLayout());
			p2.add(new Label("Number Displays: "), c);

			c.gridx = 3;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editND, c);
			p.add(p2, BorderLayout.EAST);
			big.add(p, BorderLayout.CENTER);
			
			p = new Panel(new BorderLayout());
			p2 = new Panel(new GridBagLayout());

			c.gridx = 2;
			c.anchor = GridBagConstraints.EAST;
			p2.add(new Label("Switches: "), c);

			c.gridx = 3;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editSW, c);
			p.add(p2, BorderLayout.WEST);
			
			p2 = new Panel(new GridBagLayout());
			c.gridx = 2;
			c.anchor = GridBagConstraints.EAST;
			p2.add(new Label("Push Buttons: "), c);

			c.gridx = 3;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editPB, c);
			p.add(p2, BorderLayout.EAST);
			big.add(p, BorderLayout.SOUTH);
			
			this.add(big, BorderLayout.NORTH);
			
			big = new Panel(new BorderLayout(0, 15));
			p = new Panel(new BorderLayout());
			p.add(this.clock, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);
			
			p2 = new Panel(new GridBagLayout());
			c = new GridBagConstraints();
			p = new Panel(new BorderLayout());

			c.weightx = 1;
			c.weighty = 0;
			c.gridheight = 1;
			c.fill = GridBagConstraints.NONE;

			c.insets = new Insets(0, 0, 0, 0);
			c.gridwidth = 1;
			c.gridy = 1;
			c.anchor = GridBagConstraints.EAST;
			p2.add(new Label("From"), c);

			c.gridx = 1;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editFrom, c);
			p.add(p2,BorderLayout.WEST);

			c.gridx = 2;
			c.anchor = GridBagConstraints.EAST;
			
			p2 = new Panel(new GridBagLayout());
			p2.add(new Label("Hz to"), c);

			c.gridx = 3;
			c.anchor = GridBagConstraints.WEST;
			p2.add(this.editTo, c);
			p.add(p2, BorderLayout.CENTER);
			p.add(new Label("Hz"), BorderLayout.EAST);
			big.add(p, BorderLayout.CENTER);
			
			this.add(big, BorderLayout.CENTER);
		}

		public void ok() {
			VHDLExportChoices.LED_COLOR = this.oldLED;
			VHDLExportChoices.SW_COLOR = this.oldSW;
			VHDLExportChoices.PB_COLOR = this.oldPB;
			VHDLExportChoices.ND_COLOR = this.oldND;
			VHDLExportChoices.FROM = this.oldfrom;
			VHDLExportChoices.TO = this.oldto;
		}

		public void actionPerformed(ActionEvent e) {
			double newFrom;
			double newTo;
			
			try
			{
				newFrom = Double.valueOf(this.editFrom.getText()).doubleValue();
				newTo = Double.valueOf(this.editTo.getText()).doubleValue();
				
				if(newFrom < 0)
					this.editFrom.setText(Double.toString(this.oldfrom));
				else
					this.oldfrom = newFrom;
				
				if(newTo < 0)
					this.editTo.setText(Double.toString(this.oldto));
				else
					this.oldto = newTo;
			}
			catch(NumberFormatException nfe)
			{
				this.editFrom.setText(Double.toString(this.oldfrom));
				this.editTo.setText(Double.toString(this.oldto));
			}
		}

		public void adjustmentValueChanged(AdjustmentEvent e) {
		}

		public void itemStateChanged(ItemEvent e) {
			String selected;

			if (e.getSource() == this.editLED) {
				selected = this.editLED.getSelectedItem();
				this.oldLED = VHDLExportChoices.getColor(selected);
			} else if (e.getSource() == this.editSW) {
				selected = this.editSW.getSelectedItem();
				this.oldSW = VHDLExportChoices.getColor(selected);
			} else if (e.getSource() == this.editPB) {
				selected = this.editPB.getSelectedItem();
				this.oldPB = VHDLExportChoices.getColor(selected);
			} else {
				selected = this.editND.getSelectedItem();
				this.oldND = VHDLExportChoices.getColor(selected);
			}
		}

		public void focusGained(FocusEvent e) {
			if (e.getSource() == this.editLED) {
				this.editLED.setBackground(Color.white);
			} else if (e.getSource() == this.editSW) {
				this.editSW.setBackground(Color.white);
			} else if (e.getSource() == this.editPB) {
				this.editPB.setBackground(Color.white);
			} else if (e.getSource() == this.editND) {
				this.editND.setBackground(Color.white);
			}
		}

		public void focusLost(FocusEvent e) {
			String selected;
			double newFrom;
			double newTo;

			try
			{
				newFrom = Double.valueOf(this.editFrom.getText()).doubleValue();
				newTo = Double.valueOf(this.editTo.getText()).doubleValue();
				
				if(newFrom < 0)
					this.editFrom.setText(Double.toString(this.oldfrom));
				else {
					this.oldfrom = newFrom;
					this.editFrom.setText(this.editFrom.getText());
				}
				
				if(newTo < 0)
					this.editTo.setText(Double.toString(this.oldto));
				else {
					this.oldto = newTo;
					this.editTo.setText(this.editTo.getText());
				}
			}
			catch(NumberFormatException nfe)
			{
				this.editFrom.setText(Double.toString(this.oldfrom));
				this.editTo.setText(Double.toString(this.oldto));
			}

			if (e.getSource() == this.editLED) {
				selected = this.editLED.getSelectedItem();
				this.oldLED = VHDLExportChoices.getColor(selected);
				this.editLED.setBackground(this.oldLED);
			} else if (e.getSource() == this.editSW) {
				selected = this.editSW.getSelectedItem();
				this.oldSW = VHDLExportChoices.getColor(selected);
				this.editSW.setBackground(this.oldSW);
			} else if (e.getSource() == this.editPB) {
				selected = this.editPB.getSelectedItem();
				this.oldPB = VHDLExportChoices.getColor(selected);
				this.editPB.setBackground(this.oldPB);
			} else if (e.getSource() == this.editND) {
				selected = this.editND.getSelectedItem();
				this.oldND = VHDLExportChoices.getColor(selected);
				this.editND.setBackground(this.oldND);
			}
		}

		public Dimension getPreferredSize() {
			return this.getSize();
		}

		public Dimension getMinimumSize() {
			return this.getSize();
		}

		public Dimension getMaximumSize() {
			return this.getSize();
		}
	}
}
