package sim.lib.functions;

import java.awt.*;

import sim.*;
import sim.engine.*;

import sim.lib.wires.Junction;

public class NotEqual extends Function2in1thinOut
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/functions/NotEqualUnitIcon.gif");
	
	public Image getIcon()
	{
		return NotEqual.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "Not equal unit";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public NotEqual()
	{
		super();
	}
	
	protected Wrapper getCopy()
	{
		NotEqual result = new NotEqual();
		result.changeDelay(this.delay);
		result.setBusSize(this.busSize);
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			int tripple = 3 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawLine(increment, gridGap, 10 * increment - gridGap, gridGap);
			g.drawLine(14 * increment - gridGap, gridGap, 19 * increment - gridGap, gridGap);
			
			g.drawLine(10 * increment - gridGap, gridGap, twice, 6 * increment);
			g.drawLine(14 * increment - gridGap, gridGap, twice, 6 * increment);
			
			g.drawLine(increment, tripple, 19 * increment - gridGap, tripple);
			
			g.drawLine(increment, gridGap, increment, tripple);
			g.drawLine(19 * increment - gridGap, gridGap, 19 * increment - gridGap, tripple);
			
			g.fillRect(gridGap - 1, 0, 3, gridGap);
			g.fillRect(3 * gridGap - 1, 0, 3, gridGap);
			g.drawLine(twice, tripple, twice, tripple + gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("=", twice - fm.stringWidth("=") / 2, 10 * increment);
			g.drawString("/", twice - fm.stringWidth("/") / 2, 10 * increment);
		}
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		boolean foundUndefined = false;
		boolean notEqual = false;
		
		for(int loop = 0; (loop < this.busSize) && (!foundUndefined); loop++)
		{
			if(currentInputs[loop].isUndefined() || currentInputs[loop + this.busSize].isUndefined())
				foundUndefined = true;
			else
				notEqual = notEqual || (currentInputs[loop].getValue() ^ currentInputs[loop + this.busSize].getValue());
		}
		
		if(foundUndefined)
			peer.setOutputPinUndefined(0, time);
		else
			peer.setOutputPinValue(0, notEqual, time);
	}
	
	@Override
	public void writeVHDLEntity(String path) {
		VHDLGeneral.write2busto1bitFunction(path,getVHDLName(),this.busSize,"x /= y");
	}

	@Override
	public String getVHDLName() {
		return "EQUALbus_ent_" + this.busSize;
	}
}