package sim.lib.memory;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import javax.swing.JOptionPane;

import sim.*;
import sim.engine.*;
import sim.util.SimSeparator;
import sim.lib.wires.Junction;

public class RSflipflop extends WrapperPainted implements EngineModule
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/memory/RSIcon.gif");
	
	public Image getIcon()
	{
		return RSflipflop.ICON;
	}
	
	public Wrapper createWrapper()
	{
		return this.getCopy();
	}
	
	public Wrapper createWrapper(Point gridPosition)
	{
		RSflipflop result = this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}
	
	public String getBubbleHelp()
	{
		return "RS flip flop";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public RSflipflop()
	{
		super();
		this.valueQ = 0;
	}
	
	protected RSflipflop getCopy()
	{
		RSflipflop result = new RSflipflop();
		
		result.changeDelay(this.delay);
		result.setInvertR(this.invertR);
		result.setInvertS(this.invertS);
		result.setLabel(this.getLabel());
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void initializeGridSize()
	{
		this.setGridSize(7, 6);
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int diameter = 2 * increment;
			int tripple = 3 * gridGap;
			int four = 4 * gridGap;
			int five = 5 * gridGap;
			int x = gridGap;
			int y = gridGap;
			int width = 7 * gridGap;
			int height = 6 * gridGap;
			int center = x + width / 2;
			Shape old = g.getClip();

			FontMetrics fm;
			
			int end = 2 * gridGap - increment;
			
			width = 6 * gridGap;
			x = center - width / 2;
			y = 0;
			
			g.setColor(this.brush);
			
			g.setClip(x, y, width, height);

			g.setFont(new Font(Wrapper.FONT_NAME,Font.PLAIN, gridGap - 2));
			fm = g.getFontMetrics(g.getFont());
			x = center - 3 * fm.stringWidth(this.name) / 4;
			y = 2 * gridGap / 3;
			g.drawString(this.name, x, y);

			g.setClip(old);
			
			g.drawRect(end, 3 * diameter / 2, four + increment, five);
			
			g.drawLine(6 * gridGap, gridGap, 7 * gridGap, gridGap);
			g.drawLine(6 * gridGap, five, 7 * gridGap, five);
			
			if(this.invertR)
			{
				g.drawLine(0, gridGap, end - diameter - 1, gridGap);
				g.drawOval(end - diameter - 1, gridGap - increment, diameter, diameter);
			}
			else
				g.drawLine(0, gridGap, end, gridGap);
			
			if(this.invertS)
			{
				g.drawLine(0, five, end - diameter - 1, five);
				g.drawOval(end - diameter - 1, five - increment, diameter, diameter);
			}
			else
				g.drawLine(0, five, end, five);
			
			//draws current value of flipflop
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 6 * increment));
			g.setColor(Color.white);
			if(this.pastQ == -2)
				g.drawString("X", tripple + increment * 2, tripple + increment);
			else
				g.drawString(Integer.toString(this.pastQ), tripple + increment * 2, tripple + increment);
			g.setColor(this.brush);
			if(this.valueQ == -2)
				g.drawString("X", tripple + increment * 2, tripple + increment);
			else
				g.drawString(Integer.toString(this.valueQ), tripple + increment * 2, tripple + increment);
			
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			g.drawString("R", 2 * gridGap + increment, gridGap + diameter);
			g.drawString("S", 2 * gridGap + increment, five);
			
			g.drawString("Q", five, gridGap + diameter);
			
			increment = g.getFontMetrics(g.getFont()).getAscent();
			g.drawLine(five, five - increment, five + g.getFontMetrics(g.getFont()).stringWidth("Q"), five - increment);
			g.drawString("Q", five, five);
		}
	}
	
/* ==================================================================
	Maintanance Part
	================================================================= */
	private Junction inputR = null;
	private Junction inputS =null;
	private Junction outputQ = null;
	private Junction outputNotQ = null;
	
	protected String name = "";
	
	private boolean invertR = false;
	private boolean invertS = false;
	
	public void setInvertR(boolean invert)
	{
		this.invertR = invert;
	}
	
	public void setInvertS(boolean invert)
	{
		this.invertS = invert;
	}
	
	public void setLabel(String newName)
	{
		if(newName == null) {
			this.name = "";
		} else {
			this.name = newName;
		}
	}
	
	public String getLabel()
	{
		return this.name;
	}
	
	public boolean canDrop()
	{
		boolean result = Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 1, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 5, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 7, this.gridLocation.y + 1, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 7, this.gridLocation.y + 5, 1);
		
		return result;
	}
		
	public void droped()
	{
		this.inputR = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 1, 1);
		this.inputS = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 5, 1);
		this.outputQ = Wrapper.setPinAt(this.gridLocation.x + 7, this.gridLocation.y + 1, 1);
		this.outputNotQ = Wrapper.setPinAt(this.gridLocation.x + 7, this.gridLocation.y + 5, 1);
		
		this.changeColor(Color.black);
	}
	
	public void selected()
	{
		this.inputR.removePin();
		this.inputS.removePin();
		this.outputQ.removePin();
		this.outputNotQ.removePin();
		
		this.changeColor(Color.green);
	}
	
	public void checkAfterSelected()
	{
		Wrapper.checkPin(this.inputR);
		Wrapper.checkPin(this.inputS);
		Wrapper.checkPin(this.outputQ);
		Wrapper.checkPin(this.outputNotQ);
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	protected double delay = 1;
	protected int valueQ;
	protected int pastQ;
	
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		if(currentInputs[0].isUndefined() || currentInputs[1].isUndefined())
		{
			peer.setOutputPinUndefined(0, time);
			peer.setOutputPinUndefined(1, time);
			
			this.pastQ = this.valueQ;
			this.valueQ = -1;
		}
		else if((!currentInputs[0].getValue() ^ this.invertR) && (!currentInputs[1].getValue() ^ this.invertS))
		{
			if(this.valueQ == -1 || this.valueQ == -2)
			{
				peer.setOutputPinUndefined(0, time);
				peer.setOutputPinUndefined(1, time);
			}
		}
		else if((currentInputs[0].getValue() ^ this.invertR) && (currentInputs[1].getValue() ^ this.invertS))
		{
			this.pastQ = this.valueQ;
			this.valueQ = -2;
			if(currentTime != 0)
				JOptionPane.showMessageDialog(null, "R and S shouldn't be set to high at the same time", "WARNING", JOptionPane.WARNING_MESSAGE);
			peer.setOutputPinValue(0, false, time);
			peer.setOutputPinValue(1, false, time);
		}
		else if(currentInputs[0].getValue() ^ this.invertR)
		{
			this.pastQ = this.valueQ;
			this.valueQ = 0;
			peer.setOutputPinValue(0, false, time);
			peer.setOutputPinValue(1, true, time);
		}
		else if(currentInputs[1].getValue() ^ this.invertS)
		{
			this.pastQ = this.valueQ;
			this.valueQ = 1;
			peer.setOutputPinValue(0, true, time);
			peer.setOutputPinValue(1, false, time);
		}

		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}
	
	public void createEnginePeer(EnginePeerList epl)
	{
		EnginePeer ep = new EnginePeer(2, 2, this);

		ep.setInputPin(0, this.inputR.getNodes().getItemAt(0));
		ep.setInputPin(1, this.inputS.getNodes().getItemAt(0));
		
		ep.setOutputPin(0, this.outputQ.getNodes().getItemAt(0));
		ep.setOutputPin(1, this.outputNotQ.getNodes().getItemAt(0));
		
		this.pastQ = this.valueQ;
		
		if(this.valueQ == 0) {
			ep.setOutputPinValue(0, false, 0);
			ep.setOutputPinValue(1, true, 0);
		} else {
			ep.setOutputPinValue(0, true, 0);
			ep.setOutputPinValue(1, false, 0);
		}
		
		epl.insertItem(ep);
	}
	
	public void reset()
	{
	}
	
	public Wrapper getParentWrapper()
	{
		return this;
	}

	public double getDelay()
	{
		return this.delay;
	}
	
	public void changeDelay(double newValue)
	{
		this.delay = newValue;
	}
	
/* ==================================================================
	Storage Part
	================================================================= */
	public String getSpecificParameters()
	{
		return (this.name + Wrapper.SEPARATOR + Double.toString(this.delay) + Wrapper.SEPARATOR + this.invertR + Wrapper.SEPARATOR + this.invertS + Wrapper.SEPARATOR);
	}
	
	public void loadWrapper(String[] specificParameters) throws SimException
	{
		if(specificParameters.length == this.getNumberOfSpecificParameters())
		{
			try
			{
				this.setLabel(specificParameters[0]);
				this.delay = Double.valueOf(specificParameters[1]).doubleValue();
				this.setInvertR(Boolean.valueOf(specificParameters[2]).booleanValue());
				this.setInvertS(Boolean.valueOf(specificParameters[3]).booleanValue());
			}
			catch(NumberFormatException e)
			{
				throw (new SimException("incorrect parameter type"));
			}
		}
		else
			throw (new SimException("incorrect number of parameters"));
	}
	
	public int getNumberOfSpecificParameters()
	{
		return 4;
	}
	
/* ==================================================================
	Popup Part
	================================================================= */
	public boolean hasProperties()
	{
		return true;
	}
	
	public Component getPropertyWindow()
	{
		return (new RSproperties(this.delay, this.name, this.invertR, this.invertS, valueQ));
	}
		
	public void respondToChanges(Component property)
	{
		CentralPanel.ACTIVE_GRID.eraseComponent(this, false);
		
		RSproperties x = (RSproperties)property;
		this.name = x.getComponentName();
		this.delay = x.getDelay();
		this.invertR = x.getInvertR();
		this.invertS = x.getInvertS();
		valueQ = x.getValue();
		
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}
	
	private class RSproperties extends Container implements ActionListener, FocusListener
	{
		private TextField editComponentName = new TextField();
		private TextField editDelay = new TextField(10);
		private Checkbox editR = new Checkbox("Invert R");
		private Checkbox editS = new Checkbox("Invert S");
		private Choice editValue = new Choice();
		private double old;
		private String oldComponentName;
		private String oldName;

		private Label componentName = new Label("Name");
		private Label pins = new Label("Pins");
		private Label simulation = new Label("Simulation");
		
		public RSproperties(double initial, String compName, boolean invertR, boolean invertS, int value)
		{
			super();
			this.setLayout(new BorderLayout(0, 15));

			this.oldComponentName = compName;
			this.editComponentName.setText(this.oldComponentName);
			this.editR.setState(invertR);
			this.editS.setState(invertS);
			this.editValue.add("0");
			this.editValue.add("1");
			if(value == -1|| value == -2)
				this.editValue.select(0);
			else
				this.editValue.select(value);
			this.old = initial;
			
			this.editComponentName.addActionListener(this);
			this.editComponentName.addFocusListener(this);
			this.editDelay.addActionListener(this);
			this.editDelay.addFocusListener(this);
			this.editDelay.setText(Double.toString(initial));
			
			Panel big = new Panel(new BorderLayout(0, 15));
			Panel p = new Panel(new BorderLayout());

			p.add(this.componentName, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);
			p = new Panel(new GridLayout(1, 1));
			p.add(new Label("Component Name"));
			p.add(this.editComponentName);
			big.add(p, BorderLayout.CENTER);
			
			this.add(big, BorderLayout.NORTH);

			big = new Panel(new BorderLayout(0, 15));
			p = new Panel(new BorderLayout());
			p.add(this.pins, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);
			
			p = new Panel(new GridLayout(1, 2));
			p.add(this.editR);
			p.add(this.editS);
			big.add(p, BorderLayout.CENTER);
			
			this.add(big, BorderLayout.CENTER);
			
			
			big = new Panel(new BorderLayout(0, 15));
			p = new Panel(new BorderLayout());
			p.add(this.simulation, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);

			p = new Panel(new GridLayout(1, 1));
			p.add(new Label("Propagation Delay"));
			p.add(this.editDelay);
			big.add(p, BorderLayout.CENTER);
			
			p = new Panel(new GridLayout(1, 1));
			p.add(new Label("Value     "));
			p.add(this.editValue);
			big.add(p, BorderLayout.SOUTH);
			
			this.add(big, BorderLayout.SOUTH);
		}
		
		public void addNotify()
		{
			super.addNotify();
			this.setSize(290, this.editComponentName.getPreferredSize().height + this.editR.getPreferredSize().height * 2 + this.editDelay.getPreferredSize().height + this.pins.getPreferredSize().height * 3 + this.editValue.getPreferredSize().height + 60);
		}
		
		public void actionPerformed(ActionEvent e)
		{
			double newDelay;
			String newComponentName;
			
			try
			{
				newDelay = Double.valueOf(this.editDelay.getText()).doubleValue();
				newComponentName = this.editComponentName.getText();
				
				if(newDelay < 0)
					this.editDelay.setText(Double.toString(this.old));
				else
					this.old = newDelay;
				
				if(newComponentName.length() == 0)
					this.editComponentName.setText(this.oldName);
				else
					this.oldName = newComponentName;
			}
			catch(NumberFormatException nfe)
			{
				this.editDelay.setText(Double.toString(this.old));
				this.editComponentName.setText(this.oldName);
			}
			valueQ = Integer.valueOf(this.editValue.getSelectedItem()).intValue();
		}
		
		public void focusGained(FocusEvent e)
		{
		}
		
		public void focusLost(FocusEvent e)
		{
			double newDelay;
			String newComponentName;
			
			try
			{
				newDelay = Double.valueOf(this.editDelay.getText()).doubleValue();
				newComponentName = this.editComponentName.getText();
				
				if(newDelay < 0)
					this.editDelay.setText(Double.toString(this.old));
				else
				{
					this.old = newDelay;
					this.editDelay.setText(this.editDelay.getText());
				}
				
				if(newComponentName.length() == 0)
					this.editComponentName.setText(this.oldName);
				else 
				{
					this.oldName = newComponentName;
					this.editComponentName.setText(this.editComponentName.getText());
				}
			}
			catch(NumberFormatException nfe)
			{
				this.editDelay.setText(Double.toString(this.old));
				this.editComponentName.setText(this.oldName);
			}
			valueQ = Integer.valueOf(this.editValue.getSelectedItem()).intValue();
		}
		
		public double getDelay()
		{
			double newDelay;
			
			try
			{
				newDelay = Double.valueOf(this.editDelay.getText()).doubleValue();
				
				if(newDelay >= 0)
					this.old = newDelay;
			}
			catch(NumberFormatException nfe)
			{
			}
			
			return this.old;
		}
		
		public String getComponentName() {
			this.oldName = this.editComponentName.getText();

			this.editComponentName.setText(this.oldName);

			return this.oldName;
		}
		
		public boolean getInvertR()
		{
			return this.editR.getState();
		}
		
		public boolean getInvertS()
		{
			return this.editS.getState();
		}
		
		public int getValue()
		{
			return Integer.valueOf(this.editValue.getSelectedItem()).intValue();
		}
		
		public Dimension getPreferredSize()
		{
			return this.getSize();
		}
		
		public Dimension getMinimumSize()
		{
			return this.getSize();
		}
	
		public Dimension getMaximumSize()
		{
			return this.getSize();
		}
	}

	@Override
	public void writeVHDLEntity(String path) {
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port(\n");
				writer.write("r,s : in std_logic;\n");
				writer.write("q : out std_logic;\n");
				writer.write("notq : out std_logic);\n");
				writer.write("end "+ this.getVHDLName() + ";\n\n");
				
				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("signal temp: std_logic;\n");
				writer.write("begin\n");
				writer.write("process (r,s,temp) is\n");
				writer.write("begin\n");

				writer.write("if (r ='"+(this.invertR ? 1:0)+"' and s='"+(this.invertS ? 1:0)+"') then\n");
				writer.write("temp <= temp;\n");
				writer.write("q <= temp;\n");
				writer.write("notq <= not temp;\n");
				writer.write("elsif (r='"+(this.invertR ? 1:0)+"' and s='"+(this.invertS ? 0:1)+"') then\n");
				writer.write("temp <= '1';\n");
				writer.write("q <= '1'\n");
				writer.write("notq <= '0';\n");
				writer.write("elsif (r='"+(this.invertR ? 0:1)+"' and s='"+(this.invertS ? 1:0)+"') then\n");
				writer.write("temp <= '0';\n");
				writer.write("q <= '0';\n");
				writer.write("notq <= '1';\n");
				writer.write("elsif (r='"+(this.invertR ? 0:1)+"' and s='"+(this.invertS ? 0:1)+"') then\n");
				writer.write("temp <= '0';\n");
				writer.write("q <= '0';\n");
				writer.write("notq <= '0';\n");
				writer.write("end if;\n");
				writer.write("end if;\n");
				writer.write("end process;\n");
				writer.write("end behav;\n");
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	@Override
	public String getVHDLName() {
		return "RSflipflop";
	}
}
