package sim.lib.sorces;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.util.Scanner;

import gjt.BulletinLayout;

import sim.*;
import sim.engine.*;
import sim.lib.wires.Junction;
import sim.util.LightButton;
import sim.util.SimSeparator;
import sim.util.StickyLightButtonListenerForButton;


public class Button extends RotatableFlippableWrapperPainted implements EngineModule
{
/* ==================================================================
	Creation Part
	================================================================= */
	public Image getIcon()
	{
		return GuiFileLink.getImage("sim/lib/sorces/ButtonIcon.gif");
	}
	
	public Wrapper createWrapper() 
	{
		Button result = new Button();
		result.setupWrapper();
		return result;
	}
	
	public Wrapper createWrapper(Point gridPosition)
	{
		Button result = new Button();
		result.setupWrapper();
		result.setGridLocation(gridPosition);
		return result;
	}
	
	public String getBubbleHelp()
	{
		return "Button";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	private LightButton button = null;
	
	public Button()
	{
		super();
	}
	
	protected Wrapper getCopy()
	{
		Button result = new Button();
		
		result.setupWrapper();
		result.invert = this.invert;
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void setupWrapper()
	{
		int gap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		
		this.button = new LightButton(gap, gap);
		this.button.setLocation(3 * gap / 2, gap+ 3);
		this.button.setBackground(WrapperPainted.BACKGROUND);
		this.button.setEnabled(true);
		
		this.button.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if(peer != null)
						if(!Button.invert)
							peer.normalTransaction(0, new Signal(button.isInset(), RunShortcut.LINK.getRealTime(), false, peer, 0));
						else
							peer.normalTransaction(0, new Signal(button.isRaised(), RunShortcut.LINK.getRealTime(), false, peer, 0));
					
					CentralPanel.ACTIVE_GRID.paintComponent(Button.this);
				}
			});
		
		
		StickyLightButtonListenerForButton listener = new StickyLightButtonListenerForButton();
		this.button.addMouseListener(listener);
		this.button.addMouseMotionListener(listener);
		
		this.setLayout(new BulletinLayout());
		this.add(this.button);
	}
	
	public void initializeGridSize()
	{
		this.setGridSize(4, 4);
	}
	
	public void scale()
	{
		int gap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		
		this.button.setSize(gap, gap);
		this.button.setLocation(3 * gap / 2, gap + 3);
	}
	
/* ==================================================================
	Maintanance Part
	================================================================= */
	private Junction output = null;
	public static boolean invert = false;
	
	public void selected()
	{
		this.output.removePin();
		
		this.changeColor(Color.green);
	}
	
	public void checkAfterSelected()
	{
		Wrapper.checkPin(this.output);
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	private EnginePeer peer = null;
	
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
	}
	
	public void createEnginePeer(EnginePeerList epl)
	{
		this.peer = new EnginePeer(0, 1, this);
		
		this.peer.setOutputPin(0, this.output.getNodes().getItemAt(0));
		if(!Button.invert)
			this.peer.normalTransaction(0, new Signal(this.button.isInset(), 0, false, this.peer, 0));
		else
			this.peer.normalTransaction(0, new Signal(this.button.isRaised(), 0, false, this.peer, 0));
		
		CentralPanel.ACTIVE_GRID.paintComponent(this);
		
		epl.insertItem(this.peer);
	}
	
	public void reset()
	{
		this.peer = null;
	}
	
	public Wrapper getParentWrapper()
	{
		return this;
	}
	
/* ==================================================================
	Storage Part
	================================================================= */
	public String getSpecificParameters()
	{
		
		return ((new Boolean(this.button.isRaised())).toString() + Wrapper.SEPARATOR + this.invert + Wrapper.SEPARATOR);
	}
	
	public void loadWrapper(String[] specificParameters) throws SimException
	{
		this.setupWrapper();
		
		if(specificParameters.length == this.getNumberOfSpecificParameters())
		{
			if(Boolean.valueOf(specificParameters[0]).booleanValue()) {
				this.button.setRaised();
			} else
				this.button.setInset();
		}
		else
			throw (new SimException("incorrect number of parameters"));
	}
	
	public int getNumberOfSpecificParameters()
	{
		return 2;
	}
/* ==================================================================
	Popup Part
	================================================================= */
	public boolean hasProperties()
	{
		return true;
	}
	
	public Component getPropertyWindow()
	{
		return (new Buttonproperties(Button.invert));
	}
		
	public void respondToChanges(Component property)
	{
		CentralPanel.ACTIVE_GRID.eraseComponent(this, true);
		
		Buttonproperties x = (Buttonproperties)property;
		Button.invert = x.getInvert();
		
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}
	
	private class Buttonproperties extends Container implements ActionListener, FocusListener
	{
		private Checkbox edit = new Checkbox("Invert");
		
		private Label pins = new Label("Pins");
		
		public Buttonproperties(boolean invert)
		{
			super();
			this.setLayout(new BorderLayout(0, 15));
			
			this.edit.setState(invert);
			
			Panel big = new Panel(new BorderLayout(0, 15));
			Panel p = new Panel(new BorderLayout());
			p.add(this.pins, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);
			
			p = new Panel(new GridLayout(1, 2));
			p.add(this.edit);
			big.add(p, BorderLayout.CENTER);
			
			this.add(big, BorderLayout.NORTH);
			
			this.add(big, BorderLayout.CENTER);
		}
		
		public void addNotify()
		{
			super.addNotify();
			this.setSize(290, this.edit.getPreferredSize().height + this.pins.getPreferredSize().height + 45);
		}
		
		public void actionPerformed(ActionEvent e)
		{
		}
		
		public void focusGained(FocusEvent e)
		{
		}
		
		public void focusLost(FocusEvent e)
		{
		}
		
		public boolean getInvert()
		{
			return this.edit.getState();
		}
		
		public Dimension getPreferredSize()
		{
			return this.getSize();
		}
		
		public Dimension getMinimumSize()
		{
			return this.getSize();
		}
	
		public Dimension getMaximumSize()
		{
			return this.getSize();
		}
	}
	
/* ==================================================================
	Rotation abd Flipping Part
	================================================================= */
	
	protected void paintNormal_0(Graphics g)
	{
		int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		int middle = 2 * gridGap;
		int right = 3 * gridGap;
		int bottom = 3 * gridGap;
		int increment = gridGap / 4;
		
		g.setColor(WrapperPainted.BACKGROUND);
		g.fillRect(gridGap, gridGap, 2 * bottom / 3, middle);
		g.fill3DRect(5 * increment, 9 * increment, 3 * increment + 12 , 2 * increment + 2, false);
		
		g.setColor(WrapperPainted.BACKGROUND.brighter());
		g.drawLine(gridGap + 1, gridGap + 1, right - 2, gridGap + 1);
		g.drawLine(gridGap + 1, gridGap + 1, gridGap + 1, bottom - 2);
		
		g.setColor(WrapperPainted.BACKGROUND.darker());
		g.drawLine(right - 1, gridGap + 2, right - 1, bottom - 2);
		g.drawLine(gridGap + 1, bottom - 2, right - 2, bottom - 2);
		
		g.setColor(this.brush);
		g.drawLine(right, middle, right + gridGap, middle);
		g.drawRect(gridGap, gridGap, 2 * bottom / 3, middle - 1);
		
		Rectangle buttonBounds = this.button.getBounds();
		Graphics bg = g.create(buttonBounds.x, buttonBounds.y, buttonBounds.width, buttonBounds.height);
		this.button.paint(bg);
		bg.dispose();
		
		if(this.peer != null)
		{
			if((this.button.isRaised() && !Button.invert) || (this.button.isInset() && Button.invert))
				g.setColor(SimulationProperties.ZERO_COLOR);
			else
				g.setColor(SimulationProperties.ONE_COLOR);
			
			g.fillRect(5 * increment + 1, 9 * increment + 1, 3 * increment + 10, 2 * increment);
		}
	}
	
	protected void paintNormal_90(Graphics g)
	{
		int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		int middle = 2 * gridGap;
		int right = 3 * gridGap;
		int bottom = 3 * gridGap;
		int increment = gridGap / 4;
		
		g.setColor(WrapperPainted.BACKGROUND);
		g.fillRect(gridGap, gridGap, 2 * bottom / 3, middle);
		g.fill3DRect(5 * increment, 9 * increment, 3 * increment + 12 , 2 * increment + 2, false);
		
		g.setColor(WrapperPainted.BACKGROUND.brighter());
		g.drawLine(gridGap + 1, gridGap + 1, right - 2, gridGap + 1);
		g.drawLine(gridGap + 1, gridGap + 1, gridGap + 1, bottom - 2);
		
		g.setColor(WrapperPainted.BACKGROUND.darker());
		g.drawLine(right - 1, gridGap + 2, right - 1, bottom - 2);
		g.drawLine(gridGap + 1, bottom - 2, right - 2, bottom - 2);
		
		g.setColor(this.brush);
		g.drawLine(bottom - gridGap, 0, bottom - gridGap, gridGap);
		g.drawRect(gridGap, gridGap, 2 * bottom / 3, middle - 1);
		
		Rectangle buttonBounds = this.button.getBounds();
		Graphics bg = g.create(buttonBounds.x, buttonBounds.y, buttonBounds.width, buttonBounds.height);
		this.button.paint(bg);
		bg.dispose();
		
		if(this.peer != null)
		{
			if((this.button.isRaised() && !Button.invert) || (this.button.isInset() && Button.invert))
				g.setColor(SimulationProperties.ZERO_COLOR);
			else
				g.setColor(SimulationProperties.ONE_COLOR);
			
			g.fillRect(5 * increment + 1, 9 * increment + 1, 3 * increment + 10, 2 * increment);
		}
	}
	
	protected void paintNormal_180(Graphics g)
	{
		int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		int middle = 2 * gridGap;
		int right = 3 * gridGap;
		int bottom = 3 * gridGap;
		int increment = gridGap / 4;
		
		g.setColor(WrapperPainted.BACKGROUND);
		g.fillRect(gridGap, gridGap, 2 * bottom / 3, middle);
		g.fill3DRect(5 * increment, 9 * increment, 3 * increment + 12 , 2 * increment + 2, false);
		
		g.setColor(WrapperPainted.BACKGROUND.brighter());
		g.drawLine(gridGap + 1, gridGap + 1, right - 2, gridGap + 1);
		g.drawLine(gridGap + 1, gridGap + 1, gridGap + 1, bottom - 2);
		
		g.setColor(WrapperPainted.BACKGROUND.darker());
		g.drawLine(right - 1, gridGap + 2, right - 1, bottom - 2);
		g.drawLine(gridGap + 1, bottom - 2, right - 2, bottom - 2);
		
		g.setColor(this.brush);
		g.drawLine(0, middle, gridGap, middle);
		g.drawRect(gridGap, gridGap, 2 * bottom / 3, middle - 1);
		
		Rectangle buttonBounds = this.button.getBounds();
		Graphics bg = g.create(buttonBounds.x, buttonBounds.y, buttonBounds.width, buttonBounds.height);
		this.button.paint(bg);
		bg.dispose();
		
		if(this.peer != null)
		{
			if((this.button.isRaised() && !Button.invert) || (this.button.isInset() && Button.invert))
				g.setColor(SimulationProperties.ZERO_COLOR);
			else
				g.setColor(SimulationProperties.ONE_COLOR);
			
			g.fillRect(5 * increment + 1, 9 * increment + 1, 3 * increment + 10, 2 * increment);
		}
	}
	
	protected void paintNormal_270(Graphics g)
	{
		int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		int middle = 2 * gridGap;
		int right = 3 * gridGap;
		int bottom = 3 * gridGap;
		int increment = gridGap / 4;
		
		g.setColor(WrapperPainted.BACKGROUND);
		g.fillRect(gridGap, gridGap, 2 * bottom / 3, middle);
		g.fill3DRect(5 * increment, 9 * increment, 3 * increment + 12 , 2 * increment + 2, false);
		
		g.setColor(WrapperPainted.BACKGROUND.brighter());
		g.drawLine(gridGap + 1, gridGap + 1, right - 2, gridGap + 1);
		g.drawLine(gridGap + 1, gridGap + 1, gridGap + 1, bottom - 2);
		
		g.setColor(WrapperPainted.BACKGROUND.darker());
		g.drawLine(right - 1, gridGap + 2, right - 1, bottom - 2);
		g.drawLine(gridGap + 1, bottom - 2, right - 2, bottom - 2);
		
		g.setColor(this.brush);
		g.drawLine(bottom - gridGap, bottom, bottom - gridGap, bottom + gridGap);
		g.drawRect(gridGap, gridGap, 2 * bottom / 3, middle - 1);
		
		Rectangle buttonBounds = this.button.getBounds();
		Graphics bg = g.create(buttonBounds.x, buttonBounds.y, buttonBounds.width, buttonBounds.height);
		this.button.paint(bg);
		bg.dispose();
		
		if(this.peer != null)
		{
			if((this.button.isRaised() && !Button.invert) || (this.button.isInset() && Button.invert))
				g.setColor(SimulationProperties.ZERO_COLOR);
			else
				g.setColor(SimulationProperties.ONE_COLOR);
			
			g.fillRect(5 * increment + 1, 9 * increment + 1, 3 * increment + 10, 2 * increment);
		}
	}
	
	protected void paintFlipped_0(Graphics g)
	{
		this.paintNormal_0(g);
	}
	
	protected void paintFlipped_90(Graphics g)
	{
		this.paintNormal_270(g);
	}
	
	protected void paintFlipped_180(Graphics g)
	{
		this.paintNormal_180(g);
	}
	
	protected void paintFlipped_270(Graphics g)
	{
		this.paintNormal_90(g);
	}
	
	
	
	
	protected boolean canDropNormal_0()
	{
		return Wrapper.canDropJuncion(this.gridLocation.x + 4, this.gridLocation.y + 2, 1);
	}
	
	protected boolean canDropNormal_90()
	{
		return Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y, 1);
	}
	
	protected boolean canDropNormal_180()
	{
		return Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 2, 1);
	}
	
	protected boolean canDropNormal_270()
	{
		return Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y + 4, 1);
	}
	
	protected boolean canDropFlipped_0()
	{
		return this.canDropNormal_0();
	}
	
	protected boolean canDropFlipped_90()
	{
		return this.canDropNormal_270();
	}
	
	protected boolean canDropFlipped_180()
	{
		return this.canDropNormal_180();
	}
	
	protected boolean canDropFlipped_270()
	{
		return this.canDropNormal_90();
	}
	
	
	
	
	protected void dropedNormal_0()
	{
		this.output = Wrapper.setPinAt(this.gridLocation.x + 4, this.gridLocation.y + 2, 1);
		this.changeColor(Color.black);
	}
	
	protected void dropedNormal_90()
	{
		this.output = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y, 1);
		this.changeColor(Color.black);
	}
	
	protected void dropedNormal_180()
	{
		this.output = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 2, 1);
		this.changeColor(Color.black);
	}
	
	protected void dropedNormal_270()
	{
		this.output = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y + 4, 1);
		this.changeColor(Color.black);
	}
	
	protected void dropedFlipped_0()
	{
		this.dropedNormal_0();
	}
	
	protected void dropedFlipped_90()
	{
		this.dropedNormal_270();
	}
	
	protected void dropedFlipped_180()
	{
		this.dropedNormal_180();
	}
	
	protected void dropedFlipped_270()
	{
		this.dropedNormal_90();
	}
		
	
	
	
	protected void adjustToChanges()
	{
	}
	
	public void writeVHDLEntity(String path)
	{
		String fileName = "testbench.vhd";
		File file = new File(path+"\\"+fileName);
		FileWriter writer = null;
		try {
			writer = new FileWriter(path+"\\"+fileName,true);
			writer.write(" := '" + (this.invert ^ VHDLExportChoices.PB_COLOR == SimulationProperties.ONE_COLOR ? 0 : 1) + "';\n");
			
			writer.close();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String getVHDLName()
	{
		return "Button";
	}	
}